@@ -145,20 +145,17 @@ PrimeInstance::process_key_event_lookup_keybind (const KeyEvent& key)
 bool
 PrimeInstance::process_remaining_key_event (const KeyEvent &key)
 {
-    if (key.mask & SCIM_KEY_ControlMask ||
-        key.mask & SCIM_KEY_Mod1Mask ||
-        key.mask & SCIM_KEY_Mod2Mask ||
-        key.mask & SCIM_KEY_Mod3Mask ||
-        key.mask & SCIM_KEY_Mod4Mask ||
-        key.mask & SCIM_KEY_Mod5Mask)
-    {
-        return false;
+    if (!is_registering () && isspace (key.get_ascii_code ())) {
+        if (is_preediting ())
+            return true;
+        else
+            return false;
     }
 
-    if (!is_registering () && isspace (key.get_ascii_code ()))
-        return false;
-
-    if (get_session () && isprint (key.get_ascii_code ())) {
+    if (get_session () &&
+        isprint (key.get_ascii_code ()) &&
+        !isspace (key.get_ascii_code ()))
+    {
         bool was_preediting = is_preediting ();
 
         if (is_converting () ||
@@ -193,7 +190,10 @@ PrimeInstance::process_remaining_key_event (const KeyEvent &key)
         return false;
     }
 
-    return false;
+    if (is_preediting ())
+        return true;
+    else
+        return false;
 }
 
 void
@@ -463,6 +463,8 @@ PrimeInstance::set_preedition (void)
         }
 
         if (!m_candidates.empty ()) {
+            // on inline prediction mode
+
             if (!left.empty () && cursor.empty () && right.empty ())
                 cursor = utf8_mbstowcs (" ");
 
@@ -487,8 +489,30 @@ PrimeInstance::set_preedition (void)
             update_aux_string (left + cursor + right, attr_list);
 
         } else {
-            update_preedit_string (left + cursor + right);
-            update_preedit_caret (left.length ());
+            // on normal prediction mode
+
+            bool place_cursor_head
+                =  m_factory->m_predict_on_preedition
+                && m_factory->m_predict_win_pos == "head";
+
+            if (place_cursor_head && !left.empty () &&
+                cursor.empty () && right.empty ())
+            {
+                cursor = utf8_mbstowcs (" ");
+            }
+
+            AttributeList attr_list;
+            if (!cursor.empty ()) {
+                Attribute attr (left.length (), cursor.length(),
+                                SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
+                attr_list.push_back (attr);
+            }
+
+            update_preedit_string (left + cursor + right, attr_list);
+            if (place_cursor_head)
+                update_preedit_caret (0);
+            else
+                update_preedit_caret (left.length ());
 
             hide_aux_string ();
             update_aux_string (utf8_mbstowcs (""));
@@ -576,16 +600,18 @@ PrimeInstance::set_preedition_on_register (void)
             get_session()->conv_predict (m_candidates);
         }
 
-        attr.set_type (SCIM_ATTR_DECORATE);
-        attr.set_value (SCIM_ATTR_DECORATE_HIGHLIGHT);
-        attr.set_start (pos);
-
         if (!m_candidates.empty ()) {
+
             // preedition
             preedition = m_candidates[0].m_conversion;
             str += preedition;
 
+            attr.set_type (SCIM_ATTR_DECORATE);
+            //attr.set_value (SCIM_ATTR_DECORATE_HIGHLIGHT);
             attr.set_value (SCIM_ATTR_DECORATE_REVERSE);
+            attr.set_start (pos);
+            attr.set_length (preedition.length ());
+            attr_list.push_back (attr);
 
             // reading
             if (!left.empty () && cursor.empty () && right.empty ())
@@ -603,17 +629,43 @@ PrimeInstance::set_preedition_on_register (void)
             update_aux_string (reading, reading_attr_list);
 
         } else {
-            pos += left.length ();
-            reading = left + cursor + right;
-            preedition = reading;
+            if (m_factory->m_predict_on_preedition &&
+                m_factory->m_predict_win_pos == "head")
+            {
+                if (!left.empty () && cursor.empty () && right.empty ())
+                    cursor = utf8_mbstowcs (" ");
+
+                attr.set_type (SCIM_ATTR_DECORATE);
+                attr.set_value (SCIM_ATTR_DECORATE_HIGHLIGHT);
+                attr.set_start (pos);
+                attr.set_length (left.length ());
+                attr_list.push_back (attr);
+
+                attr.set_value (SCIM_ATTR_DECORATE_REVERSE);
+                attr.set_start (pos + left.length ());
+                attr.set_length (cursor.length ());
+                attr_list.push_back (attr);
+
+                attr.set_value (SCIM_ATTR_DECORATE_HIGHLIGHT);
+                attr.set_start (pos + left.length () + cursor.length ());
+                attr.set_length (right.length ());
+                attr_list.push_back (attr);
+            } else {
+                pos += left.length ();
+
+                attr.set_type (SCIM_ATTR_DECORATE);
+                attr.set_value (SCIM_ATTR_DECORATE_HIGHLIGHT);
+                attr.set_start (pos);
+                attr.set_length (left.length () + cursor.length () + right.length ());
+                attr_list.push_back (attr);
+            }
+
+            preedition = left + cursor + right;
             str += preedition;
 
             hide_aux_string ();
             update_aux_string (utf8_mbstowcs (""));
         }
-
-        attr.set_length (preedition.length ());
-        attr_list.push_back (attr);
     }
 
     str += m_registering_value.substr (
